"""Command line utility to convert SQLite to NetCDF"""

import argparse
import os
import sqlite3
import sys
from datetime import datetime
import netCDF4 as nc
import numpy as np


def arguments():
    """Function captures the command-line arguments passed to this script"""

    description = """
    Program for converting .db file format to NetCDF format.

    For help, contact John.Krasting@noaa.gov
    """

    parser = argparse.ArgumentParser(
        description=description, formatter_class=argparse.RawTextHelpFormatter
    )

    parser.add_argument(
        "infile", type=str, help="Input file. Format must be sqlite (*.db)"
    )

    parser.add_argument(
        "-o",
        "--outfile",
        type=str,
        default="out.nc",
        help="Output file. Default name is out.nc",
    )

    parser.add_argument(
        "-F",
        "--force",
        action="store_true",
        default=False,
        help="Clobber existing output file if it exists.",
    )

    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        default=False,
        help="Verbose output. Default is quiet output.",
    )

    return parser.parse_args()


def check_file(filepath, clobber=False):
    """
    Checks if output file exists. If clobber set to True, the existing file
    is removed
    """
    if os.path.exists(filepath):
        if clobber is True:
            os.remove(filepath)
        else:
            raise ValueError(
                "FATAL: output file exits. Trying using the "
                + "'-F' option to force overwrite."
            )


def tables_and_years(dbfile):
    """
    Function to get list of variables and years in a .db file
    """
    conn = sqlite3.connect(dbfile)
    cur = conn.cursor()
    sql = "SELECT name FROM sqlite_master WHERE type='table'"
    _ = cur.execute(sql)
    tables = [str(record[0]) for record in cur.fetchall()]
    cur.close()
    conn.close()

    years = []
    for table in tables:
        if table not in ["long_name", "units", "cell_measure"]:
            conn = sqlite3.connect(dbfile)
            cur = conn.cursor()
            sql = "SELECT year FROM " + table
            try:
                _ = cur.execute(sql)
            except RuntimeError as error:
                sys.stderr.write("Unable to process " + table)
                sys.stderr.write("Variable " + table + " does not contain year axis")
                raise error
            data = [int(record[0]) for record in cur.fetchall()]
            cur.close()
            conn.close()
            years = years + data

    years = list(set(years))

    return tables, years


def write_nc(
    dbfile,
    outfile,
    tables,
    years,
    clobber=False,
    ncformat="NETCDF3_CLASSIC",
    verbose=False,
):
    """Writes output to a netCDF file"""
    check_file(outfile, clobber=clobber)
    dt_obj = datetime.now()
    timestamp_str = dt_obj.strftime("%d-%b-%Y (%H:%M:%S.%f)")
    ncfile = nc.Dataset(outfile, "w", format=ncformat)
    ncfile.setncattr("source_file", dbfile)
    ncfile.setncattr("created", timestamp_str)
    _ = ncfile.createDimension("time", 0)
    time = ncfile.createVariable("time", "f4", ("time",))
    time.calendar = "noleap"
    time.units = "days since 0001-01-01 00:00:00.0"
    time[:] = ((np.array(years) - 1) * 365.0) + 196.0
    for table in tables:
        if table not in ["long_name", "units", "cell_measure"]:
            data_array = np.ma.ones(len(years)) + 1.0e20
            data_array.mask = True
            extract_list = ["value"]
            for k in extract_list:
                count = 0
                conn = sqlite3.connect(dbfile)
                cur = conn.cursor()

                cur.execute(f"SELECT value from long_name where var='{table}'")
                result = cur.fetchone()
                long_name = result[0] if result is not None else ""

                cur.execute(f"SELECT value from units where var='{table}'")
                result = cur.fetchone()
                units = result[0] if result is not None else ""

                if verbose is True:
                    print("Processing %s  :  %s" % (table, long_name))
                for year in years:
                    sql = "SELECT " + k + " FROM " + table + " where year=" + str(year)
                    _ = cur.execute(sql)
                    data = [float(record[0]) for record in cur.fetchall()]
                    if data:
                        data_array[count] = data[0]
                        data_array.mask[count] = False
                    else:
                        pass
                    count = count + 1
                cur.close()
                conn.close()
                if len(extract_list) > 1:
                    outname = table + "_" + k.replace("sum", "int")
                else:
                    outname = table
            var = ncfile.createVariable(outname, "f4", ("time"))
            if long_name != "":
                var.long_name = long_name
            if units != "":
                var.units = units
            var[:] = data_array
    ncfile.close()


def main():
    """Entry point for the db2nc command"""
    args = arguments()
    infile = os.path.realpath(args.infile)
    _tables, _years = tables_and_years(infile)
    write_nc(
        infile, args.outfile, _tables, _years, clobber=args.force, verbose=args.verbose
    )


if __name__ == "__main__":
    main()



"""CLI script for plotting SQLite files"""

import argparse
import os

import gfdlvitals


def arguments():
    """
    Function to capture the user-specified command line options
    """
    description = """
    Program for plotting global mean statistics

    For help, contact John.Krasting@noaa.gov

    """

    parser = argparse.ArgumentParser(
        description=description, formatter_class=argparse.RawTextHelpFormatter
    )

    parser.add_argument(
        "dbfiles",
        nargs="+",
        metavar="DB FILES",
        type=str,
        default=os.getcwd(),
        help="Path to input database files",
    )

    parser.add_argument(
        "-a",
        "--align",
        action="store_true",
        default=False,
        help="Align different time axes",
    )

    parser.add_argument(
        "-t",
        "--trend",
        action="store_true",
        default=False,
        help="Add trend lines to plots",
    )

    parser.add_argument(
        "-s",
        "--smooth",
        type=int,
        default=None,
        help="Apply a n-years smoother to all plots",
    )

    parser.add_argument(
        "-l",
        "--labels",
        type=str,
        default=None,
        help="Comma-separated list of dataset labels",
    )

    parser.add_argument(
        "-n",
        "--nyears",
        type=int,
        default=None,
        help="Limit the plotting to a set number of n years",
    )

    args = parser.parse_args()
    return args


def main():
    """Entry point for the plotdb command"""
    cliargs = arguments()
    gfdlvitals.plot.run_plotdb(cliargs)


if __name__ == "__main__":
    main()



""" Extract scalar fields from ocean model output """

import numpy as np
from . import gmeantools

__all__ = ["mom6"]


def mom6(fdata, fyear, outdir, outname="globalAveOcean.db"):
    """Extract MOM6 scalar output and save to sqlite

    Parameters
    ----------
    fdata : netCDF4.Dataset
        Input ocean_scalar.nc file
    fyear : str
        Year being processed
    outdir : str
        Path to output directory
    """

    ignore_list = ["time_bounds", "time_bnds", "average_T2", "average_T1", "average_DT", "nv"]

    var_dict = fdata.variables.keys()
    var_dict = list(set(var_dict) - set(ignore_list))

    for varname in var_dict:
        if len(fdata.variables[varname].shape) <= 2:
            units = gmeantools.extract_metadata(fdata, varname, "units")
            long_name = gmeantools.extract_metadata(fdata, varname, "long_name")
            result = fdata.variables[varname]
            if result.shape[0] == 12:
                if len(result.shape) == 2:
                    result = result[:, 0]
                else:
                    result = result[:]
                result = np.ma.average(
                    result,
                    weights=[
                        31.0,
                        28.0,
                        31.0,
                        30.0,
                        31.0,
                        30.0,
                        31.0,
                        31.0,
                        30.0,
                        31.0,
                        30.0,
                        31.0,
                    ],
                    axis=0,
                )
            elif result.shape[0] == 1:
                if len(result.shape) == 2:
                    result = result[0, 0]
                else:
                    result = result[0]
            sqlfile = f"{outdir}/{fyear}.{outname}"
            gmeantools.write_metadata(sqlfile, varname, "units", units)
            gmeantools.write_metadata(sqlfile, varname, "long_name", long_name)
            gmeantools.write_sqlite_data(sqlfile, varname, fyear[:4], result)



"""Basic tests for gfdlvitals package"""

import os


def test_import():
    import gfdlvitals


def test_version_exists():
    import gfdlvitals
    assert isinstance(gfdlvitals.__version__, str)
    assert len(gfdlvitals.__version__) > 0


def test_submodule_imports():
    from gfdlvitals import averagers
    from gfdlvitals import cli
    from gfdlvitals import models
    from gfdlvitals import sample
    from gfdlvitals import util


def test_vitals_dataframe():
    from gfdlvitals import VitalsDataFrame
    df = VitalsDataFrame({"a": [1, 2, 3]})
    assert len(df) == 3


def test_sample_files_exist():
    from gfdlvitals import sample
    assert os.path.isfile(sample.historical)
    assert os.path.isfile(sample.picontrol)


def test_open_db():
    from gfdlvitals import open_db, sample, VitalsDataFrame
    df = open_db(sample.historical)
    assert isinstance(df, VitalsDataFrame)
    assert len(df) > 0



""" Sample db files for demonstration """

from importlib.resources import files

historical = str(files("gfdlvitals").joinpath("resources/historical.db"))
picontrol = str(files("gfdlvitals").joinpath("resources/picontrol.db"))
